@@ -1,6 +1,7 @@
-use std::collections::BTreeMap;
+use proc_macro2::{Span, TokenStream};
+use quote::{quote, ToTokens};
 
-use quote::quote;
+use std::collections::BTreeMap;
 
 use crate::attrs::ExportScope;
 use crate::function::{
@@ -9,26 +10,33 @@ use crate::function::{
 };
 use crate::module::Module;
 
-pub type ExportedConst = (String, Box<syn::Type>, syn::Expr);
+pub type ExportedConst = (String, Box<syn::Type>, syn::Expr, Vec<syn::Attribute>);
 
 pub fn generate_body(
     fns: &mut [ExportedFn],
     consts: &[ExportedConst],
     sub_modules: &mut [Module],
     parent_scope: &ExportScope,
-) -> proc_macro2::TokenStream {
-    let mut set_fn_statements: Vec<syn::Stmt> = Vec::new();
-    let mut set_const_statements: Vec<syn::Stmt> = Vec::new();
-    let mut add_mod_blocks: Vec<syn::ExprBlock> = Vec::new();
-    let mut set_flattened_mod_blocks: Vec<syn::ExprBlock> = Vec::new();
+) -> TokenStream {
+    let mut set_fn_statements = Vec::new();
+    let mut set_const_statements = Vec::new();
+    let mut add_mod_blocks = Vec::new();
+    let mut set_flattened_mod_blocks = Vec::new();
     let str_type_path = syn::parse2::<syn::Path>(quote! { str }).unwrap();
     let string_type_path = syn::parse2::<syn::Path>(quote! { String }).unwrap();
 
-    for (const_name, _, _) in consts {
-        let const_literal = syn::LitStr::new(&const_name, proc_macro2::Span::call_site());
-        let const_ref = syn::Ident::new(&const_name, proc_macro2::Span::call_site());
+    for (const_name, _, _, cfg_attrs) in consts {
+        let const_literal = syn::LitStr::new(&const_name, Span::call_site());
+        let const_ref = syn::Ident::new(&const_name, Span::call_site());
+
+        let cfg_attrs: Vec<_> = cfg_attrs
+            .iter()
+            .map(syn::Attribute::to_token_stream)
+            .collect();
+
         set_const_statements.push(
             syn::parse2::<syn::Stmt>(quote! {
+                #(#cfg_attrs)*
                 m.set_var(#const_literal, #const_ref);
             })
             .unwrap(),
@@ -41,15 +49,8 @@ pub fn generate_body(
             continue;
         }
         let module_name = item_mod.module_name();
-        let exported_name: syn::LitStr = syn::LitStr::new(
-            item_mod.exported_name().as_ref(),
-            proc_macro2::Span::call_site(),
-        );
-        let cfg_attrs: Vec<&syn::Attribute> = item_mod
-            .attrs()
-            .iter()
-            .filter(|&a| a.path.get_ident().map(|i| *i == "cfg").unwrap_or(false))
-            .collect();
+        let exported_name = syn::LitStr::new(item_mod.exported_name().as_ref(), Span::call_site());
+        let cfg_attrs = crate::attrs::collect_cfg_attr(item_mod.attrs());
         add_mod_blocks.push(
             syn::parse2::<syn::ExprBlock>(quote! {
                 {
@@ -71,19 +72,20 @@ pub fn generate_body(
     }
 
     // NB: these are token streams, because re-parsing messes up "> >" vs ">>"
-    let mut gen_fn_tokens: Vec<proc_macro2::TokenStream> = Vec::new();
+    let mut gen_fn_tokens = Vec::new();
+
     for function in fns {
         function.update_scope(&parent_scope);
         if function.skipped() {
             continue;
         }
         let fn_token_name = syn::Ident::new(
-            &format!("{}_token", function.name().to_string()),
+            &format!("{}_token", function.name()),
             function.name().span(),
         );
         let reg_names = function.exported_names();
 
-        let fn_input_types: Vec<syn::Expr> = function
+        let fn_input_types: Vec<_> = function
             .arg_list()
             .map(|fn_arg| match fn_arg {
                 syn::FnArg::Receiver(_) => panic!("internal error: receiver fn outside impl!?"),
@@ -126,6 +128,12 @@ pub fn generate_body(
             })
             .collect();
 
+        let cfg_attrs: Vec<_> = function
+            .cfg_attrs()
+            .iter()
+            .map(syn::Attribute::to_token_stream)
+            .collect();
+
         for fn_literal in reg_names {
             let mut namespace = FnNamespaceAccess::Internal;
 
@@ -166,6 +174,7 @@ pub fn generate_body(
 
             set_fn_statements.push(
                 syn::parse2::<syn::Stmt>(quote! {
+                    #(#cfg_attrs)*
                     m.set_fn(#fn_literal, FnNamespace::#ns_str, FnAccess::Public,
                              #param_names, &[#(#fn_input_types),*], #fn_token_name().into());
                 })
@@ -174,9 +183,11 @@ pub fn generate_body(
         }
 
         gen_fn_tokens.push(quote! {
+            #(#cfg_attrs)*
             #[allow(non_camel_case_types)]
             pub struct #fn_token_name();
         });
+
         gen_fn_tokens.push(function.generate_impl(&fn_token_name.to_string()));
     }
 
@@ -229,8 +240,8 @@ pub fn check_rename_collisions(fns: &[ExportedFn]) -> Result<(), syn::Error> {
             })
     }
 
-    let mut renames = BTreeMap::<String, proc_macro2::Span>::new();
-    let mut fn_defs = BTreeMap::<String, proc_macro2::Span>::new();
+    let mut renames = BTreeMap::new();
+    let mut fn_defs = BTreeMap::new();
 
     for item_fn in fns.iter() {
         if !item_fn.params().name.is_empty() || item_fn.params().special != FnSpecialAccess::None {
@@ -251,11 +262,11 @@ pub fn check_rename_collisions(fns: &[ExportedFn]) -> Result<(), syn::Error> {
                 if let Some(other_span) = renames.insert(key, current_span) {
                     let mut err = syn::Error::new(
                         current_span,
-                        format!("duplicate Rhai signature for '{}'", &fn_name),
+                        format!("duplicate Rhai signature for '{}'", fn_name),
                     );
                     err.combine(syn::Error::new(
                         other_span,
-                        format!("duplicated function renamed '{}'", &fn_name),
+                        format!("duplicated function renamed '{}'", fn_name),
                     ));
                     return Err(err);
                 }
@@ -263,13 +274,11 @@ pub fn check_rename_collisions(fns: &[ExportedFn]) -> Result<(), syn::Error> {
         } else {
             let ident = item_fn.name();
             if let Some(other_span) = fn_defs.insert(ident.to_string(), ident.span()) {
-                let mut err = syn::Error::new(
-                    ident.span(),
-                    format!("duplicate function '{}'", ident.to_string()),
-                );
+                let mut err =
+                    syn::Error::new(ident.span(), format!("duplicate function '{}'", ident));
                 err.combine(syn::Error::new(
                     other_span,
-                    format!("duplicated function '{}'", ident.to_string()),
+                    format!("duplicated function '{}'", ident),
                 ));
                 return Err(err);
             }
@@ -277,11 +286,11 @@ pub fn check_rename_collisions(fns: &[ExportedFn]) -> Result<(), syn::Error> {
             if let Some(fn_span) = renames.get(&key) {
                 let mut err = syn::Error::new(
                     ident.span(),
-                    format!("duplicate Rhai signature for '{}'", &ident),
+                    format!("duplicate Rhai signature for '{}'", ident),
                 );
                 err.combine(syn::Error::new(
                     *fn_span,
-                    format!("duplicated function '{}'", &ident),
+                    format!("duplicated function '{}'", ident),
                 ));
                 return Err(err);
             }
